program mkm3u;

{$APPTYPE CONSOLE}

uses
  SysUtils;

var
    i,age : integer;
    deleteM3U,verbose : Boolean;

procedure errorExit(msg:String);

var
  basename : String;
  i : integer;
begin
  i := LastDelimiter('\',ParamStr(0));
  if i > 0  then basename:=Copy(ParamStr(0),i+1, 255);
  i := LastDelimiter('.',basename);
  if i > 0  then basename:=Copy(basename,0,i-1);
  writeln(basename+': '+msg);
  writeln('Usage: '+basename+' [-a ndays] [-v] [-d] mp3-root-directory' );
  writeln(' -a <n>   : create extra m3u-playlists which get prefixed by an underscore.' );
  writeln('            only files which are newer than <n> days get added.' );
  writeln(' -v       : verbose reporting.' );
  writeln(' -d       : delete all existing m3u files before creating new ones' );
  writeln;
  writeln('mkm3u is a utility to generate m3u files from mp3-directories.' );
  Halt(1);
end;

procedure processDir(dirname:String);

var
  sr: TSearchRec;
  M3U : Text;
  s : String;
begin
  s:='';
  if verbose then writeln('processing '+dirname);
  if (age > 0) then
  begin
    if ( FindFirst(dirname+'\.', faDirectory, sr) = 0 ) then
      if ( (Date - FileDateToDateTime(sr.Time)) < age ) then s:='_';
    FindClose(sr);
  end;
  if ( (FindFirst(dirname+'\*.m3u', 0, sr) = 0) and deleteM3U ) then
  repeat
    if verbose then writeln( 'delete '+dirname+'\'+sr.Name );
    deleteFile(dirname+'\'+sr.Name);
  until FindNext(sr) <> 0;
  FindClose(sr);
  if FindFirst(dirname+'\*.mp3', 0, sr) = 0 then
  begin
    s:=dirname+'\'+s+ExtractFileName(dirname)+'.m3u';
    if verbose then writeln( 'create '+s);
    AssignFile(M3U,s);
    {$I-}
    Rewrite(M3U);
    {$I+}
    if IOResult <> 0 then writeln('Cannot write file '+s)
    else
    begin
      repeat
        writeln(M3U, sr.Name );
      until FindNext(sr) <> 0;
      CloseFile(M3U);
    end;

  end;
  FindClose(sr);
  if FindFirst(dirname+'\*.*', faDirectory, sr) = 0 then
  repeat
    if (((sr.Attr and faDirectory) > 0) and (sr.Name <> '.') and (sr.Name <> '..'))
    then processDir(dirname+'\'+sr.Name);
  until FindNext(sr) <> 0;
  FindClose(sr);
end;


begin
  age:=0;
  verbose:=False;
  deleteM3U:=False;
  if ParamCount < 1 then errorExit('not enough parameters');
  i:=0;
  while ( i < ParamCount ) do
  begin
    if ParamStr(i) = '-v' then verbose := True;
    if ParamStr(i) = '-d' then deleteM3U := True;
    if ParamStr(i) = '-a' then
    begin
        if ( i = ParamCount-1 ) then errorExit('invalid syntax');
        age:=StrToIntDef(ParamStr(i+1), -1);
        if ( age < 0 ) then errorExit('invalid age: '+ParamStr(i+1));
        i:=i+1;
    end;
    i:=i+1;
  end;
  processDir(ParamStr(ParamCount));
  //readln;
end.
